<!--
// © University of Southampton IT Innovation Centre, 2017
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road, 
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Michael Boniface
//      Updated By :            Simon Crowle
//      Created Date :          18-12-2017
//      Update Date :           14-02-2018
//      Created for Project :   FLAME
-->

# CLMC Adaptive Streaming Test

This test streams mpeg-dash video using the two nginx servers monitored by Telegraf configured with a default apache plugin and a net_response plugin. The data is stored in the `clmc-service` using database `CLMCMetrics` and measurements `nginx` and `net_response`

The following command brings up the services

`vagrant --fixture=streaming up`

* clmc-service: configured with influx, kapacitor, chornograf
* nginx1@DC1, nginx2@DC2: configured with nginx and a test video located at http://192.168.50.11:80/test_video/stream.mpd on the internal vbox network and at http://localhost:8081/test_video/stream.mpd if accessing from the host machine

### Run the test set-up

`vagrant --fixture=streaming ssh clmc-service -- "sudo /vagrant/test/streaming/setupCLMC.sh /vagrant/test/streaming"`  
`vagrant --fixture=streaming ssh nginx1 -- "sudo /vagrant/test/streaming/setupNGINX.sh"`

### Run the automated test

To run the load test using the following command (here, the last parameter '15' refers to the number of VLC player clients to be launched):

`vagrant --fixture=streaming ssh loadtest-streaming -- "sudo /vagrant/test/streaming/run.sh /home/ubuntu/test/streaming http://192.168.50.11/test_video/stream.mpd 15`

This test currently just generates the load and does not have any assertions. It breaks at 1000.

And then point your browser to the Chronograf dashboard:

`http://localhost:8888`

### Run the automated PyTests

SSH into the clmc-service VM:

`vagrant --fixture=streaming ssh clmc-service`

Run the automated tests written in pytest:

`pytest -s /vagrant/test/streaming/` 

### Manual test 

## Manual set-up of Chronograf's CLMC data source

If you __do not__ want to run the automatic set-up, basic entry to the Chronograf dashboard is as follows:

1. Point your browser to: [http://localhost:8888](http://localhost:8888)
2. Enter your connection string: `http://localhost:8086`
3. Enter the Name: `Influx 1`
4. Enter the Telegraf database: `CLMCMetrics`

## Manual test on Windows

### View the video
Install VLC video client on the host machine, you must use a very recent version otherwise the MPD file cannot we read. At the time of writng the following nighly build was installed:

https://nightlies.videolan.org/build/win32/vlc-3.0.0-rc1-20171201-0326/vlc-3.0.0-20171201-0326-rc1-win32.exe

Start the VLC Player

`Media->Open Network Stream`

The test video is the FLAME project video and it can be viewed at the following location.

`Enter the network URL: http://localhost:8081/test_video/stream.mpd for nginx1 server`

The video should play.

### Query the data

Open Chronograph by entering the following URL into a browser on the host http://localhost:8888. Your CLMC data source, Kapacitor and demonstration dashboard should be ready for you to explore.

Press the Data Explorer in the menu and select the nginx measurement and create a query such as 

`SELECT mean("requests") AS "mean_requests" FROM "CLMCMetrics"."autogen"."nginx" WHERE time > now() - 1h GROUP BY time(10s)`

## KPI triggers

In this demonstrator an example KPI rule has been set up in Kapacitor which fires when the average requests per second on the Apache 1 or Apache2 server goes above certain thresholds ( a 'warning' at 0.2 requests/second and a 'critical' message at 0.5 requests/second ). The TICKscript specification for this rule is as follows:

```
dbrp "CLMCMetrics"."autogen"

// Nginx 1 rule
// -------------
var n1Data = batch
    |query(''' SELECT mean("active") AS "mean_active" FROM "CLMCMetrics"."autogen"."nginx" WHERE "ipendpoint"='adaptive_streaming_I1_nginx1' ''')
        .period(5s)
        .every(5s)

varn n1Alert = n1Data
    |alert()
        .id('{{ .Name }}/adaptive_streaming_I1_nginx1')
        .message('{{ .ID }} is {{ .Level }} Mean active connections: {{ index .Fields "mean_active" }}')
        .warn(lambda: "mean_active" > 10)
        .slack()
        .log( '/tmp/RPSLoad.log' )

// Nginx 2 rule
// -------------
var n2Data = batch
    |query(''' SELECT mean("active") AS "mean_active" FROM "CLMCMetrics"."autogen"."nginx" WHERE "ipendpoint"='adaptive_streaming_I1_nginx2' ''')
        .period(5s)
        .every(5s)

var n2Alert = n2Data
    |alert()
        .id('{{ .Name }}/adaptive_streaming_I1_nginx2')
        .message('{{ .ID }} is {{ .Level }} Mean active connections: {{ index .Fields "mean_active" }}')
        .warn(lambda: "mean_active" > 10)
        .slack()
        .log( '/tmp/RPSLoad.log' )
```

Alerts are sent to both an internal logging within the CLMC service file system and also to a FLAME demo Slack service:

https://flamedemo-itinnov.slack.com

Alerts can be found under the '#clmc' channel. 

### Kapacitor rules in Chronograf's GUI

Additional rules can be added to this demonstrator either via the Chronograf GUI (see [here](https://docs.influxdata.com/chronograf/v1.4/introduction/getting-started/#4-connect-chronograf-to-kapacitor) for more information) or by using the Kapacitor HTTP API and TICKscript (for an introduction, [look here](https://docs.influxdata.com/kapacitor/v1.4/tick/)).
